@@ -8,22 +8,26 @@ Pattern Pattern::interval(States &states, Pattern &regex,
     size_t lower = interval.lower();
     size_t upper = interval.upper();
 
-    if (lower == 0 && upper == numeric_limits<size_t>::max())
+    if (lower == 0 && upper == max<size_t>())
         ret = star(states, regex);
 
     else if (lower == 0 && upper == 1)
         ret = questionMark(states, regex);
 
-    else if (lower == 1 && upper == numeric_limits<size_t>::max())
+    else if (lower == 1 && upper == max<size_t>())
         ret = plus(states, regex);
 
     else if (lower <=  upper)
+    {
         ret = copy(states, regex, lower, upper);
+        if (lower == upper && regex.fixedLength())
+            ret.d_length = lower * regex.d_length;
+    }
 
     else
     {
-        States::Pair pair = states.next2();
-        states[pair.first] = State::factory(State::EMPTY, pair.second, 0);
+        Pair pair = states.next2();
+        states[pair.first] = State(EMPTY, pair.second, 0);
 
         ret = Pattern(pair);
     }
