@@ -17,20 +17,7 @@ class \@Base
                 // idx: rule, value: tail length (NO_INCREMENTS if no tail)
     typedef std::vector<int> VectorInt;
 
-    enum        // RuleFlagsCount Indices, see s_rfc__[]
-    {
-        RULE = 0,
-        FLAGS,
-        ACCCOUNT,
-    };
-
-    enum
-    {
-        FINAL = 1,
-        INCREMENT = 2,
-        COUNT = 4,
-        BOL = 8
-    };
+    static size_t const s_unavailable = std::numeric_limits<size_t>::max();
 
     enum 
     {
@@ -65,17 +52,15 @@ $insert 8 startCondNames
     };
 
 private:
-    struct FinData            // Info about intermediate matched rules while
-    {                           // traversing the DFA
+    struct FinalData
+    {
         size_t rule;
-        size_t matchLen;
-        size_t tailCount;
+        size_t length;
     };
-
     struct Final
     {
-        FinData atBOL;
-        FinData notAtBOL;
+        FinalData std;
+        FinalData bol;
     };
 
         // class Input encapsulates all input operations. 
@@ -83,7 +68,6 @@ private:
 $insert inputInterface
 
 protected:
-
     struct StreamStruct
     {
         std::string pushedName;
@@ -91,7 +75,6 @@ protected:
     };
 
 private:
-
     std::vector<StreamStruct>    d_streamStack;
 
     std::string     d_filename;             // name of the currently processed
@@ -99,24 +82,32 @@ private:
                                             // the name "<istream #>", where
                                             // # is the sequence number of the 
                                             // istream (starting at 1)
-    StartCondition__  d_startCondition;
-    size_t          d_state;
+    int             d_startCondition = 0;
+    int             d_lopSC = 0;
+
+    size_t          d_state = 0;
     int             d_nextState;
     std::shared_ptr<std::ostream> d_out;
-    bool            d_sawEOF;               // saw EOF: ignore tailCount
-    bool            d_atBOL;                // the matched text starts at BOL
-    std::vector<size_t> d_tailCount;         
-    Final d_final;                          // 1st for BOL rules
-                                            
+    bool            d_atBOL = true;         // the matched text starts at BOL
+    Final d_final;
+
                                             // only used interactively:
     std::istream *d_in;                     // points to the input stream
     std::shared_ptr<std::istringstream> d_line; // holds line fm d_in
     
     Input           d_input;
     std::string     d_matched;              // matched characters
+    std::string     d_lopMatched;           // matched lop-rule characters 
+    std::string::iterator d_lopIter;
+    std::string::iterator d_lopTail;
+    std::string::iterator d_lopEnd;
+
+    size_t          d_lopPending;           // # pending input chars at lop1__
     bool            d_return;               // return after a rule's action 
     bool            d_more = false;         // set to true by more()
 
+    size_t (\@Base::*d_get)() = &\@Base::getInput;
+
 protected:
     std::istream   *d_in__;
     int d_token__;                          // returned by lex__
@@ -125,7 +116,7 @@ $insert 4 debugDecl
 
 $insert 4 declarations
     static size_t  const s_ranges__[];
-    static size_t  const s_rfc__[][3];
+    static size_t  const s_rf__[][2];
 
 public:
     \@Base(\@Base const &other)             = delete;
@@ -208,20 +199,60 @@ $ignoreInteractive END      end ignored section by generator/filter.cc
     void            continue__(int ch);         // handles a transition
     void            echoCh__(size_t ch);        // echoes ch, sets d_atBOL
     void            echoFirst__(size_t ch);     // handles unknown input
-    void            inspectRFCs__();            // update d_tailCount
+    void            updateFinals__();           // update a state's Final info
     void            noReturn__();               // d_return to false
     void            print__() const;            // optionally print token
     void            pushFront__(size_t ch);     // return char to Input
     void            reset__();                  // prepare for new cycle
                                                 // next input stream:
     void            switchStream__(std::istream &in, size_t lineNr);   
+    void            lopf__(size_t tail);        // matched fixed size tail
+    void            lop1__(int lopSC);          // matched ab for a/b
+    void            lop2__();                   // matches the LOP's b tail
+    void            lop3__();                   // catch-all while matching b
+    void            lop4__();                   // matches the LOP's a head
 
 private:
+    size_t getInput();
+    size_t getLOP();
     void p_pushStream(std::string const &name, std::istream *streamPtr);
-    void determineMatchedSize(FinData const &final);
-    bool atFinalState();
+    void setMatchedSize(size_t length);
+    bool knownFinalState();
+
+    template <typename ReturnType, typename ArgType>
+    static ReturnType constexpr as(ArgType value);
+    static bool constexpr available(size_t value);
+    static StartCondition__ constexpr SC(int sc);
+    static int constexpr SC(StartCondition__ sc);
 };
 
+template <typename ReturnType, typename ArgType>
+inline ReturnType constexpr \@Base::as(ArgType value)
+{
+    return static_cast<ReturnType>(value);
+}
+
+inline bool \@Base::knownFinalState()
+{
+    return (d_atBOL && available(d_final.bol.rule)) ||
+           available(d_final.std.rule);
+}
+
+inline bool constexpr \@Base::available(size_t value)
+{   
+    return value != std::numeric_limits<size_t>::max();
+}
+
+inline \@Base::StartCondition__ constexpr \@Base::SC(int sc)
+{
+    return as<StartCondition__>(sc);
+}
+
+inline int constexpr \@Base::SC(StartCondition__ sc)
+{
+    return as<int>(sc);
+}
+
 inline std::ostream &\@Base::out()
 {
     return *d_out;
@@ -237,12 +268,6 @@ inline void \@Base::push(std::string const &str)
     d_input.reRead(str, 0);
 }
 
-inline bool \@Base::atFinalState()
-{
-    return d_final.notAtBOL.rule != std::numeric_limits<size_t>::max() || 
-            (d_atBOL && d_final.atBOL.rule != std::numeric_limits<size_t>::max());
-}
-
 inline void \@Base::setFilename(std::string const &name)
 {
     d_filename = name;
@@ -260,7 +285,7 @@ inline std::string const &\@Base::matched() const
 
 inline \@Base::StartCondition__ \@Base::startCondition() const
 {
-    return d_startCondition;
+    return SC(d_startCondition);
 }
 
 inline std::string const &\@Base::filename() const
@@ -280,7 +305,7 @@ inline size_t \@Base::length() const
 
 inline void \@Base::leave(int retValue) const
 {
-    throw static_cast<Leave__>(retValue);
+    throw as<Leave__>(retValue);
 }
 
 inline size_t \@Base::lineNr() const
@@ -295,8 +320,9 @@ inline void \@Base::more()
 
 inline void \@Base::begin(StartCondition__ startCondition)
 {
-    d_dfaBase__ = 
-        s_dfaBase__[static_cast<int>(d_startCondition = startCondition)];
+$insert 4 debug "Switching to StartCondition__ # " << as<int>(startCondition)
+    // d_state is reset to 0 by reset__()
+    d_dfaBase__ = s_dfaBase__[d_startCondition = SC(startCondition)];
 }
 
 inline size_t \@Base::state__() const
@@ -305,6 +331,11 @@ inline size_t \@Base::state__() const
 }
 
 inline size_t \@Base::get__()
+{
+    return (this->*d_get)();
+}
+
+inline size_t \@Base::getInput()
 {
     return d_input.get();
 }
@@ -322,7 +353,3 @@ inline void \@Base::noReturn__()
 $insert namespace-close
 
 #endif //  \@BASE_H_INCLUDED
-
-
-
-
